#include <sys/types.h>
#include <sys/wait.h>
#include <sys/io.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <string>

using namespace std;

// executes exec with parameters params and returns its stdout in stdoutStr
// bool if exec was successful, false otherwise
bool getStdout(const char* exec, const char* const params[], string& stdoutStr)
{
    int fds[2], pid, status;
    ssize_t readbytes;
    char buf[32];

    // create the pipe for reading stdout
    if (pipe(fds)) {
	fprintf(stderr, "Unable to create pipe: %s (%d)\n", strerror(errno), errno);
        return false;
    }

    if ((pid = fork()) == 0) {
	// callee
	// we don't read from the pipe here, we only write to it
	close(fds[0]);
	close(0); close(1); close(2);
	dup2(fds[1], 1);
        close(fds[1]);
        execvp(exec, (char* const*) params);
        // we shouldn't be here...
        fprintf(stderr, "Unable to exec: %s (%d)\n", strerror(errno), errno);
        return false;
    }
    else {
	if (pid > 0) {
	    // caller: wait for called process to end
	    // we don't write to the pipe here, we only read from it
	    close(fds[1]);
	    if (waitpid(pid, &status, 0) < 0) {
		fprintf(stderr, "Unable to wait: %s (%d)\n", strerror(errno), errno);
                close(fds[0]);
		return false;
	    }
	    if (! WIFEXITED(status)) {
		fprintf(stderr, "Process exited abnormally\n");
                close(fds[0]);
		return false;
	    }
            if (WEXITSTATUS(status) == 0) {
		// process exited normally and with return code 0
		while ((readbytes = read(fds[0], buf, sizeof(buf))) > 0) {
                    stdoutStr.append(buf, readbytes);
		}
                close(fds[0]);
                return true;
	    } else {
		fprintf(stderr, "Process exited with return code %d\n", WEXITSTATUS(status));
                close(fds[0]);
		return false;
	    }
	}
	else {
	    fprintf(stderr, "Unable to fork: %s (%d)\n", strerror(errno), errno);
            close(fds[0]);
            close(fds[1]);
            return false;
	}
    }
}

const char* const C_lspci_path = "lspci";
const char* const C_lspci_params[] = { "lspci", "-n", NULL };

const char* const C_ibase_lspci_output =
    "00:00.0 Class 0600: 1106:0601 (rev 05)\n"
    "00:01.0 Class 0604: 1106:8601\n"
    "00:07.0 Class 0601: 1106:0686 (rev 40)\n"
    "00:07.1 Class 0101: 1106:0571 (rev 06)\n"
    "00:07.2 Class 0c03: 1106:3038 (rev 1a)\n"
    "00:07.3 Class 0c03: 1106:3038 (rev 1a)\n"
    "00:07.4 Class 0680: 1106:3057 (rev 40)\n"
    "00:08.0 Class 0200: 10ec:8139 (rev 10)\n"
    "00:09.0 Class 0200: 10ec:8139 (rev 10)\n"
    "00:0a.0 Class 0200: 10ec:8139 (rev 10)\n"
    "00:0b.0 Class 0200: 10ec:8139 (rev 10)\n"
    "01:00.0 Class 0300: 1023:8500 (rev 6a)\n";

bool isIbase()
{
    string stdoutStr;
    return getStdout(C_lspci_path, C_lspci_params, stdoutStr) &&
        stdoutStr.compare(C_ibase_lspci_output) == 0;
}

const char* const C_nexcom_lspci_output =
    "00:00.0 Class 0600: 8086:2560 (rev 03)\n"
    "00:02.0 Class 0300: 8086:2562 (rev 03)\n"
    "00:1d.0 Class 0c03: 8086:24c2 (rev 02)\n"
    "00:1d.1 Class 0c03: 8086:24c4 (rev 02)\n"
    "00:1d.7 Class 0c03: 8086:24cd (rev 02)\n"
    "00:1e.0 Class 0604: 8086:244e (rev 82)\n"
    "00:1f.0 Class 0601: 8086:24c0 (rev 02)\n"
    "00:1f.1 Class 0101: 8086:24cb (rev 02)\n"
    "00:1f.3 Class 0c05: 8086:24c3 (rev 02)\n"
    "01:07.0 Class 0604: 12d8:8152\n"
    "01:0a.0 Class 0200: 8086:100e (rev 02)\n"
    "01:0b.0 Class 0200: 8086:100e (rev 02)\n"
    "01:0c.0 Class 0200: 8086:1229 (rev 10)\n"
    "01:0d.0 Class 0200: 8086:1229 (rev 10)\n";

bool isNexcom()
{
    string stdoutStr;
    return getStdout(C_lspci_path, C_lspci_params, stdoutStr) &&
        stdoutStr.compare(C_nexcom_lspci_output) == 0;
}


#define led_port 0x404d
#define led_bitmask 0x80

void setLedBit(bool bit)
{
    unsigned char old_val;
    old_val = inb(led_port);
    if (bit)
	outb(old_val & (~led_bitmask), led_port);
    else
	outb(old_val | led_bitmask, led_port);
}

#define mode_port 0x0442
#define mode_bypass 0x000f
#define mode_normal 0x0010

#define watchdog_disable_port 0x0441
#define watchdog_disable_value 0x00f
#define watchdog_enable_port 0x0443

void setLanBypass(bool mode) {
    if (mode)
	outb(mode_bypass, mode_port);
    else
	outb(mode_normal, mode_port);
}

void lanBypassWatchdogDisable() {
    outb(watchdog_disable_value, watchdog_disable_port);
}

void lanBypassWatchdogSendAlive(unsigned char interval) {
    outb(interval, watchdog_enable_port);
}
